<?php

namespace Modules\Admin\Http\Controllers;

use App\Http\Controllers\Controller;
use Illuminate\Http\RedirectResponse;
use Illuminate\Http\Request;
use Illuminate\Http\Response;
use Illuminate\Contracts\Support\Renderable;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\Validator;
use Illuminate\Database\QueryException;
use Illuminate\Support\Facades\Storage;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Hash;
use Illuminate\Support\Str;


use App\Models\User;
use App\Models\AdminDetails;
use App\Models\Companies;
use App\Models\Supliers;
use App\Models\Categories;
use App\Models\Brands;
use App\Models\Products;
use App\Models\ProductGallery;
use App\Models\GRNS;
use App\Models\Returns;
use App\Models\Orders;
use App\Models\OrderProducts;
use App\Models\OrderRequests;
use App\Models\GRNProducts;

use File;
use Mail;
use Image;
use PDF;
class SalesController extends Controller
{
    public function add_order(Request $request)
    { if($request->isMethod('get')){
        $customers = User::where('user_role','2')->with('customerDetails')->get();
        $products = Products::where('status','active')->get();
        $orderNumber = $this->generateUniqueOrderNumber();
        return view('admin::sales.add_order',['products' => $products,'orderNumber' => $orderNumber,'customers' => $customers]);
    }
    if($request->isMethod('post')){

        $request->validate([
            'sales_order_number'   => 'required',
            'customer'   => 'required',
            'order_date'   => 'required',
            'delivery_date'   => 'required',
            'shipping_address'   => 'required',
            'order_status'   => 'required',
            'sales_person'   => 'required',
            'price_method'   => 'required',
            'due_date'   => 'required',

        ]);
        $x=0;
        foreach($request->product as $pro){
            if(GRNProducts::where('grn_id',$request->batch[$x])->where('product',$pro)->value('quantity') < $request->quantity[$x]){
                return back()->with('fail', 'Some products stocks are lower than the selected quantity');
            }
            $x=$x+1;
        }

        $order = new Orders();
        $order->sales_order_number =$request->sales_order_number;
        $order->customer = $request->customer;
        $order->order_date = $request->order_date;
        $order->delivery_date = $request->delivery_date;
        $order->shipping_address = $request->shipping_address;
        $order->order_status = $request->order_status;
        $order->sales_person = $request->sales_person;
        $order->price_method = $request->price_method;
        $order->due_date = $request->due_date;
        $order->notes = $request->notes;
        $order->payment_terms = $request->payment_terms;
        $order->sub_total = $request->sub_total_input;
        $order->tax = $request->tax;
        $order->discount = $request->discount;
        $order->total = $request->order_total_input;
        $order->save();

        $c=0;
        foreach($request->product as $pro){

            $product = new OrderProducts();
            $product->order_id = $order->id;
            $product->product = $pro;
            $product->batch = $request->batch[$c];
            $product->price = $request->price[$c];
            $product->quantity = $request->quantity[$c];
            $product->save();

            $current_stock =  GRNProducts::where('grn_id',$request->batch[$c])->where('product',$pro)->value('quantity');

            $stock_update = GRNProducts::where('grn_id',$request->batch[$c])->where('product',$pro)->first();
            $stock_update->quantity = $current_stock-$request->quantity[$c];
            $stock_update->update();
            
            $c=$c+1;
        }
        if($request->order_status == "Shipped"){

            $customer_email = User::where('id', $order->customer)->value('email');
            $details  = [
                'order' => $order,
            ];
            Mail::to($customer_email)->send(new \App\Mail\OrderShipped($details));

        }

        $pdf_name =$order->sales_order_number.'_invoice.pdf';
        $pdf = PDF::loadView('admin::pdf.invoice',['order' => $order]);
        $pdf->setPaper('a4', 'portrait')->save(public_path('db_files/invoices/').$pdf_name);


        return redirect()->back()->with('success', 'Order Successfully Created. <br><a target="_blank" href="' . asset('db_files/invoices/'.$pdf_name.'') . '" class="btn btn-success mt-2">Click here to view the invoice</a>');
    }

    }
    public function all_orders(Request $request)
    { if($request->isMethod('get')){
        $orders = Orders::get();
        return view('admin::sales.all_orders',['orders' => $orders]);
    }

    }
    public function order_requests(Request $request)
    { if($request->isMethod('get')){
        $orders = OrderRequests::get();
        return view('admin::sales.order_requests',['orders' => $orders]);
    }

    }
    public function delete_order_request($id,Request $request)
    { if($request->isMethod('get')){
        OrderRequests::where('id', $id)->delete();
        return back()->with('success', 'Order Request Deleted');
    }

    }
    public function download_invoice($id,Request $request)
    { if($request->isMethod('get')){
        $order = Orders::where('id', $id)->first();
        $pdf_name =$order->sales_order_number.'_invoice.pdf';
        $pdf = PDF::loadView('admin::pdf.invoice',['order' => $order]);
        return $pdf->setPaper('a4', 'portrait')->save(public_path('db_files/invoices/').$pdf_name)->download($pdf_name);
    }

    }
    public function edit_order($id,Request $request)
    {
        if($request->isMethod('get')){
            $customers = User::where('user_role','2')->with('customerDetails')->get();
            $products = Products::where('status','active')->get();
            $order_details = Orders::where('id', $id)->first();
            $order_products = OrderProducts::where('order_id', $id)->get();
            return view('admin::sales.edit_order',['products' => $products,'customers' => $customers,'order_details' => $order_details
            ,'order_products' => $order_products]);
        }
        if($request->isMethod('post')){
    
            $request->validate([
                'sales_order_number'   => 'required',
                'customer'   => 'required',
                'order_date'   => 'required',
                'delivery_date'   => 'required',
                'shipping_address'   => 'required',
                'order_status'   => 'required',
                'sales_person'   => 'required',
                'price_method'   => 'required',
                'due_date'   => 'required',
    
            ]);
            $x=0;
            // foreach($request->product as $pro){
            //     if(GRNS::where('product',$pro)->where('id', $request->batch[$x])->value('quantity') < $request->quantity[$x]){
            //         return back()->with('fail', 'Some products stocks are lower than the selected quantity');
            //     }
            //     $x=$x+1;
            // }
    
            $order_details = Orders::where('id', $id)->first();

            $order = Orders::where('id', $id)->first();
            $order->sales_order_number =$request->sales_order_number;
            $order->customer = $request->customer;
            $order->order_date = $request->order_date;
            $order->delivery_date = $request->delivery_date;
            $order->shipping_address = $request->shipping_address;
            $order->order_status = $request->order_status;
            $order->sales_person = $request->sales_person;
            $order->price_method = $request->price_method;
            $order->due_date = $request->due_date;
            $order->notes = $request->notes;
            $order->payment_terms = $request->payment_terms;
            $order->sub_total = $request->sub_total_input;
            $order->tax = $request->tax;
            $order->discount = $request->discount;
            $order->total = $request->order_total_input;
            $order->update();
    

            $c=0;
            foreach($request->product as $pro){
                if(OrderProducts::where("order_id", $id)->where("product",$pro)->exists()){
                $product = OrderProducts::where("order_id", $id)->where("product",$pro)->first();
                $product->batch = $request->batch[$c];
                $product->price = $request->price[$c];
                $product->quantity = $request->quantity[$c];
                $product->update();
                }
                else{
                $product = new OrderProducts();
                $product->order_id = $id;
                $product->product = $pro;
                $product->batch = $request->batch[$c];
                $product->price = $request->price[$c];
                $product->quantity = $request->quantity[$c];
                $product->save();
                }
                
    
                // $current_stock =  GRNS::where('id',$request->batch[$c])->value('quantity');
    
                // $stock_update = GRNS::where('id',$request->batch[$c])->first();
                // $stock_update->quantity = $current_stock-$request->quantity[$c];
                // $stock_update->update();
                
                $c=$c+1;
            }
            

            if($request->order_status == "Shipped" && $order_details->order_status != "Shipped"){

                $customer_email = User::where('id', $order->customer)->value('email');
                $details  = [
                    'order' => $order,
                ];

                Mail::to($customer_email)->send(new \App\Mail\OrderShipped($details));
            }

            return back()->with('success', 'Order Successfully Updated');
    
        }
    
    }
    
    public function get_batches($productId)
{
    $batches = GRNProducts::where('product', $productId)->with('GRN')->get();
    return response()->json($batches);
}
private function generateUniqueOrderNumber()
{
    do {
        $orderNumber = strtoupper(Str::random(6));
        
        $exists = Orders::where('sales_order_number', $orderNumber)->exists();
    } while ($exists);
    
    return $orderNumber;
}


public function mail_test(Request $request)
{ 
    // $customer_email = User::where('id', 5)->value('email');
    // $order_request = OrderRequests::where('id', 1)->first();
    //     $details  = [
    //     'order_request' => $order_request,
    //     ];
    // Mail::to($customer_email)->send(new \App\Mail\OrderConfirmation($details));

    // $customer_email = User::where('id', 5)->value('email');
    // $order = Orders::where('id', 2)->first();
    //     $details  = [
    //         'order' => $order,
    //     ];
    // Mail::to($customer_email)->send(new \App\Mail\OrderShipped($details));

        $OTP =  rand ( 0000 , 9999 );


        $details  = [
            'otp' => $OTP,
        ];

        Mail::to("comivas382@degcos.com")->send(new \App\Mail\ConfirmEmail($details));
}
}