<?php

namespace Modules\Admin\Http\Controllers;

use App\Http\Controllers\Controller;
use Illuminate\Http\RedirectResponse;
use Illuminate\Http\Request;
use Illuminate\Http\Response;
use Illuminate\Contracts\Support\Renderable;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\Validator;
use Illuminate\Database\QueryException;
use Illuminate\Support\Facades\Storage;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Hash;

use App\Models\User;
use App\Models\AdminDetails;
use App\Models\CustomerDetails;
use App\Models\Orders;
use App\Models\GRNProducts;

use File;
use Mail;
use Image;
use PDF;
class UserController extends Controller
{
    /**
     * Display a listing of the resource.
     */
    public function login(Request $request)
    {
        if($request->isMethod('get')){
            return view('admin::user.login');
         }
         if($request->isMethod('post')){

            $request->validate([
                'email'   => 'required',
                'password'  => 'required'
            ]);

               $user_data = array(
                'email'  => $request->get('email'),
                'status'  => "active",
                'user_role'  => 1,
                'password' => $request->get('password')
               );

               if(Auth::attempt($user_data))
               {
                return redirect('admin/dashboard');
               }
               else
               {
                return back()->with('fail', 'Wrong Login Details');
               }
         }
    }
    public function dashboard()
    {
        $total_customers = User::where('user_role', 2)->count();
        $new_customers = User::where('user_role', 2)->whereMonth('created_at', date('m'))->whereYear('created_at', date('Y'))->count();
        $month_orders = Orders::whereMonth('created_at', date('m'))->whereYear('created_at', date('Y'))->count();
        $today_orders = Orders::whereDate('created_at', date('Y-m-d'))->count();
        
        $today_orders_data = Orders::whereDate('created_at', date('Y-m-d'))->get();
        $today_sells = 0;
        foreach($today_orders_data as $today_orders_data){
           $today_sells = $today_sells +  $today_orders_data->total;
        }
        
        $new_orders = Orders::orderBy('id', 'DESC')->take(6);
        
        $most_selling_products = DB::table('order_products')->select('product', DB::raw('SUM(quantity) as total_sold'))->groupBy('product')->orderByDesc('total_sold')->limit(6)->get();
        $low_stock_products = DB::table('grn_products')->select('product', DB::raw('SUM(quantity) as total_quantity'))->groupBy('product')->having('total_quantity', '<', 10)->get();
    
        return view('admin::dashboard.index',['total_customers' => $total_customers,'new_customers' => $new_customers,'month_orders' => $month_orders,'today_orders' => $today_orders
        ,'today_sells' => $today_sells,'new_orders' => $new_orders,'most_selling_products' => $most_selling_products,'low_stock_products' => $low_stock_products]);
    }
    public function add_admin(Request $request)
    { if($request->isMethod('get')){
        return view('admin::user.add_admin');
    }
    if($request->isMethod('post')){

        $request->validate([
            'name'   => 'required',
            'phone'   => 'required',
            'email'   => 'required | email | unique:users',
            "password" => "required | confirmed | min:6",
        ]);

           DB::beginTransaction();
           $user = User::create([
              "email" => $request->email,
              "password" => Hash::make($request->password),
              "user_role" => 1,
              "status" => "active"
           ]);

           $userDetails = new AdminDetails();
           $userDetails->user_id =$user->id;
           $userDetails->name = $request->name;
           $userDetails->phone = $request->phone;
           $userDetails->save();
           DB::commit();
            return back()->with('success', 'Admin Successfully Added');

    }

    }
    public function admins()
    {
        $admins = User::where('user_role','1')->get();
        return view('admin::user.admins',['admins' => $admins]);
    }
    public function deactivate_admin($id){
        $user = User::find($id);
        $user->status = "inactive";
        $user->update();
        return back()->with('success', 'Admin Deactivated');

    }
    public function activate_admin($id){
        $user = User::find($id);
        $user->status = "active";
        $user->update();
        return back()->with('success', 'Admin Activated');

    }

    public function edit_admin($id,Request $request)
    {
    if($request->isMethod('get')){
    $login_details = User::where('id',$id)->first();
    $other_details = AdminDetails::where('user_id',$id)->first();
    return view('admin::user.edit_admin', ['login_details' => $login_details,'other_details' => $other_details]);
    }
    if($request->isMethod('post')){
        $request->validate([
            'name'   => 'required',
            'phone'   => 'required',
            'email'   => 'required',
        ]);

    if(!$request->current_password == null || !$request->password == null || !$request->password_confirmation == null){
        $this->validate($request, [
            "password" => "required | confirmed | min:6",
            "current_password" => "required",
           ]);
           if (Hash::check($request->input('current_password'), User::where('id', $id)->value('password'))) {
            if(User::where("id", "=", $id)->where("email", "=", $request->email)->exists()){
                $email = $request->email;
            }
            elseif(User::where("email", "=", $request->email)->exists()){
             return back()->with('fail', 'This email is already in use');
            }
            else{
             $email = $request->email;
            }

            $userDetails =  AdminDetails::where('user_id', '=', $id)->first();;
            $userDetails->name = $request->name;
            $userDetails->phone = $request->phone;
            $userDetails->update();
            $user = User::find($id);
            $user->email = $email;
            $user->password = Hash::make($request->input('password'));
            $user->update();
            return back()->with('success', 'Admin Details Successfully  Updated');
           }
           else{
            return back()->with('fail', 'Current password is incorrect.');
        }
    }
    else{
        if(User::where("id", "=", $id)->where("email", "=", $request->email)->exists()){
            $email = $request->email;
        }
        elseif(User::where("email", "=", $request->email)->exists()){
         return back()->with('fail', 'This email is already in use');
        }
        else{
         $email = $request->email;
        }

        $userDetails =  AdminDetails::where('user_id', '=', $id)->first();;
        $userDetails->name = $request->name;
        $userDetails->phone = $request->phone;
        $userDetails->update();
        $user = User::find($id);
        $user->email = $email;
        $user->update();
        return back()->with('success', 'Admin Details Successfully  Updated');
    }


    }

    }

    public function delete_admin($id,Request $request)
    {
        User::find($id)->delete();
        AdminDetails::where('user_id', '=', $id)->delete();;
        return back()->with('success', 'Admin Deleted');
    }
 public function customers()
    {
        $customers = User::where('user_role','2')->with('customerDetails')->get();
        return view('admin::user.customers',['customers' => $customers]);
    }
     public function deleted_customers()
    {
       $customers = User::onlyTrashed()->where('user_role', '2')->with(['customerDetails' => function ($query) {
    $query->withTrashed();
}])->get();

        return view('admin::user.deleted_customers',['customers' => $customers]);
    }
    public function restore_customer($id,Request $request)
    {
        User::onlyTrashed()->where('id', $id)->restore();
        CustomerDetails::onlyTrashed()->where('user_id', $id)->restore();
        return back()->with('success', 'Customer restored');
    }
        public function remove_customer($id,Request $request)
    {
       CustomerDetails::onlyTrashed()->where('user_id', $id)->forceDelete();
        User::onlyTrashed()->where('id', $id)->forceDelete();
        return back()->with('success', 'Customer Removed.');
    }
    public function add_customer(Request $request)
{
    if($request->isMethod('get')){
        return view('admin::user.add_customer');
    }
    if($request->isMethod('post')){
    $request->validate([
        'customer_name' => 'required',
        'customer_type' => 'required',
        'phone' => 'required',
        'email' => 'required|email|unique:users,email',
        'address' => 'required',
        'password' => 'required|min:6',
    ]);

    DB::beginTransaction();
    try {
        $user = User::create([
            "email" => $request->email,
            "password" => Hash::make($request->password),
            "user_role" => 2,
            "status" => "active"
        ]);

        $userDetails = new CustomerDetails();
        $userDetails->user_id = $user->id;
        $userDetails->customer_name = $request->customer_name;
        $userDetails->customer_type = $request->customer_type;
        $userDetails->phone = $request->phone;
        $userDetails->address = $request->address;
        $userDetails->company_name = $request->company_name;
        $userDetails->vat_no = $request->vat_no;
        $userDetails->kvk_no = $request->kvk_no;
        $userDetails->country = $request->country;
        $userDetails->tax_id = $request->tax_id;
        $userDetails->payment_terms = $request->payment_terms;
        $userDetails->grade = $request->grade;
        $userDetails->discount = $request->discount;
        $userDetails->loyalty_info = $request->loyalty_info;
        $userDetails->notes = $request->notes;
        $userDetails->save();

        DB::commit();


        return back()->with('success', 'Customer Added');

    } catch (\Exception $e) {
        return back()->with('fail', 'Customer fail to add');

    }
}
}
public function edit_customer($id,Request $request)
    {
    if($request->isMethod('get')){
    $login_details = User::where('id',$id)->first();
    $other_details = CustomerDetails::where('user_id',$id)->first();
    return view('admin::user.edit_customer', ['login_details' => $login_details,'other_details' => $other_details]);
    }
    if($request->isMethod('post')){
        $request->validate([
            'customer_name' => 'required',
            'customer_type' => 'required',
            'phone' => 'required',
            'email' => 'required',
            'address' => 'required',
        ]);

    if(!$request->current_password == null || !$request->password == null || !$request->password_confirmation == null){
        $this->validate($request, [
            "password" => "required | confirmed | min:6",
            "current_password" => "required",
           ]);
           if (Hash::check($request->input('current_password'), User::where('id', $id)->value('password'))) {
            if(User::where("id", "=", $id)->where("email", "=", $request->email)->exists()){
                $email = $request->email;
            }
            elseif(User::where("email", "=", $request->email)->exists()){
             return back()->with('fail', 'This email is already in use');
            }
            else{
             $email = $request->email;
            }

            $userDetails =  CustomerDetails::where('user_id', '=', $id)->first();;
            $userDetails->customer_name = $request->customer_name;
            $userDetails->customer_type = $request->customer_type;
            $userDetails->phone = $request->phone;
            $userDetails->address = $request->address;
            $userDetails->company_name = $request->company_name;
            $userDetails->vat_no = $request->vat_no;
            $userDetails->kvk_no = $request->kvk_no;
            $userDetails->country = $request->country;
            $userDetails->tax_id = $request->tax_id;
            $userDetails->payment_terms = $request->payment_terms;
            $userDetails->grade = $request->grade;
            $userDetails->discount = $request->discount;
            $userDetails->loyalty_info = $request->loyalty_info;
            $userDetails->notes = $request->notes;
            $userDetails->update();
            $user = User::find($id);
            $user->email = $email;
            $user->password = Hash::make($request->input('password'));
            $user->update();
            return back()->with('success', 'Customer Details Successfully  Updated');
           }
           else{
            return back()->with('fail', 'Current password is incorrect.');
        }
    }
    else{
        if(User::where("id", "=", $id)->where("email", "=", $request->email)->exists()){
            $email = $request->email;
        }
        elseif(User::where("email", "=", $request->email)->exists()){
         return back()->with('fail', 'This email is already in use');
        }
        else{
         $email = $request->email;
        }

        $userDetails =  CustomerDetails::where('user_id', '=', $id)->first();;
        $userDetails->customer_name = $request->customer_name;
        $userDetails->customer_type = $request->customer_type;
        $userDetails->phone = $request->phone;
        $userDetails->address = $request->address;
        $userDetails->company_name = $request->company_name;
        $userDetails->vat_no = $request->vat_no;
        $userDetails->kvk_no = $request->kvk_no;
        $userDetails->country = $request->country;
        $userDetails->tax_id = $request->tax_id;
        $userDetails->payment_terms = $request->payment_terms;
        $userDetails->grade = $request->grade;
        $userDetails->discount = $request->discount;
        $userDetails->loyalty_info = $request->loyalty_info;
        $userDetails->notes = $request->notes;
        $userDetails->update();
        $user = User::find($id);
        $user->email = $email;
        $user->update();
        return back()->with('success', 'Customer Details Successfully  Updated');
    }


    }

    }
    public function deactivate_customer($id){
        $user = User::find($id);
        $user->status = "inactive";
        $user->update();
        return back()->with('success', 'Customer Deactivated');

    }
    public function activate_customer($id){
        $user = User::find($id);
        $user->status = "active";
        $user->update();
        return back()->with('success', 'Customer Activated');

    }
    public function delete_customer($id,Request $request)
    {
        User::find($id)->delete();
        CustomerDetails::where('user_id', '=', $id)->delete();;
        return back()->with('success', 'Customer Deleted');
    }
    function logout()
    {
     Auth::logout();
     return redirect('admin');
    }
    /**
     * Show the form for creating a new resource.
     */
    public function create()
    {
        return view('admin::create');
    }

    /**
     * Store a newly created resource in storage.
     */
    public function store(Request $request): RedirectResponse
    {
        //
    }

    /**
     * Show the specified resource.
     */
    public function show($id)
    {
        return view('admin::show');
    }

    /**
     * Show the form for editing the specified resource.
     */
    public function edit($id)
    {
        return view('admin::edit');
    }

    /**
     * Update the specified resource in storage.
     */
    public function update(Request $request, $id): RedirectResponse
    {
        //
    }

    /**
     * Remove the specified resource from storage.
     */
    public function destroy($id)
    {
        //
    }
}
