<?php

namespace App\Models;

use Spatie\Sluggable\HasSlug;
use Spatie\Sluggable\SlugOptions;
use Illuminate\Database\Eloquent\Factories\HasFactory;
use Illuminate\Database\Eloquent\Model;
use Illuminate\Database\Eloquent\SoftDeletes;

class Products extends Model
{
    use HasFactory;
    use HasSlug;
    use SoftDeletes;

    protected $table = 'products';
    protected $primaryKey = 'id';

    protected $fillable = [
        'product_name',
        'product_code',
        'parent_category',
        'sub_category',
        'suplier',
        'brand',
        'sku',
        'barcode',
        'warehouse_location',
        'units',
        'carton_size',
        'description',
        'featured_image',
        'featured_image_url',
        'status',
    ];
    public function getSlugOptions() : SlugOptions
    {
        return SlugOptions::create()
            ->generateSlugsFrom('product_name')
            ->saveSlugsTo('slug');
    }

 public function parentCategory()
    {
        return $this->belongsTo(Categories::class, 'parent_category');
    }

    // Relationship with the Sub Category
    public function subCategory()
    {
        return $this->belongsTo(Categories::class, 'sub_category');
    }

    // Relationship with the Supplier
    public function supplier()
    {
        return $this->belongsTo(Supliers::class, 'suplier');
    }

    // Relationship with the Brand
    public function brand()
    {
        return $this->belongsTo(Brands::class, 'brand');
    }

    // Relationship with the ProductGallery
    public function gallery()
    {
        return $this->hasMany(ProductGallery::class, 'product_id');
    }

    public function grn()
    {
        return $this->hasMany(GRNS::class, 'product');
    }


}
